function Query() {

	var self = this;

	var filters = [];
	var lastKey;
	var limit;
	var offset;

	function buildFilter(name, value, operator) {
		return {
			name: name,
			value: value,
			operator: operator
		};
	}

	function buildCriteria(query, operator) {
		if (query && Util.isObject(query)) {
			filters = filters.concat(self.buildQuery(query, operator));
		} else {
			filters.push(buildFilter(lastKey, query, operator));
		}
	}

	this.buildQuery = function(query, operator) {
		return Object.keys(query).map(function(key) {
			return buildFilter(key, query[key], operator);
		});
	};

	this.where = function(query) {
		if (!query) {
			return filters;
		}
		if (query && typeof(query) === "object") {
			filters = filters.concat(this.buildQuery(query, "="));
		} else {
			lastKey = query;
		}
		return this;
	};

	this.like = function(query) {
		buildCriteria(query, Query.LIKE);
		return this;
	};

	this.likeInsensitive = function likeInsensitive(query) {
		buildCriteria(query, Query.LIKE_INSENSITIVE);
		return this;
	};

	this.notLike = function(query) {
		buildCriteria(query, Query.NOT_LIKE);
		return this;
	};

	this.likeAll = function(query) {
		buildCriteria(query, Query.LIKE_ALL);
		return this;
	};

	this.mappedLikeAll = function(query) {
		buildCriteria(query, Query.MAPPED_LIKE_ALL);
		return this;
	};

	this.in = function(query) {
		buildCriteria(query, Query.IN);
		return this;
	};

	this.notIn = function(query) {
		buildCriteria(query, Query.NOT_IN);
		return this;
	};

	this.between = function(query) {
		buildCriteria(query, Query.BETWEEN);
		return this;
	};

	this.notBetween = function(query) {
		buildCriteria(query, Query.NOT_BETWEEN);
		return this;
	};

	this.notEquals = function(query) {
		buildCriteria(query, Query.NEQ);
		return this;
	};

	this.equals = function(query) {
		buildCriteria(query, Query.EQ);
		return this;
	};

	this.greater = function(query) {
		buildCriteria(query, Query.GT);
		return this;
	};

	this.greaterOrEqual = function(query) {
		buildCriteria(query, Query.GTE);
		return this;
	};

	this.lower = function(query) {
		buildCriteria(query, Query.LT);
		return this;
	};

	this.lowerOrEqual = function(query) {
		buildCriteria(query, Query.LTE);
		return this;
	};

	this.limit = function(number) {
		if (!number) {
			return limit;
		}
		if (!limit) {
			limit = number;
		} else {
			throw new TypeError("Limit already assigned! limit = " + limit);
		}
		return this;
	};

	this.offset = function(number) {
		if (!number) {
			return offset;
		}
		if (!offset) {
			offset = number;
		} else {
			throw new TypeError("Offset already assigned! offset = " + limit);
		}
		return this;
	};

	this.isNull = function() {
		buildCriteria('', Query.IS_NULL);
		return this;
	};

	this.isNotNull = function() {
		buildCriteria('', Query.IS_NOT_NULL);
		return this;
	};

	this.groupBy = function(query) {
		if (query) {
			filters.push(buildFilter(Util.parseToArray(query), null, Query.GROUP));
		}
		return this;
	};

	this.orderBy = function(query, direction) {
		if (query) {
			if (Util.isArray(query)) {
				filters.push(
					buildFilter(
						query.map(function(order) {
							return Util.isObject(order) ? order : {
								name: order,
								direction: Query.ORDER_ASC
							};
						}), null, Query.ORDER));
			} else {
				filters.push(buildFilter([{
					name: query,
					direction: direction ? direction : Query.ORDER_ASC
				}], null, Query.ORDER));
			}
		}
		return this;
	};
}

Query.LIKE = 'LIKE';
Query.LIKE_INSENSITIVE = 'LIKE_I';
Query.LIKE_ALL = 'LIKE_ALL';
Query.MAPPED_LIKE_ALL = 'MAPPED_LIKE_ALL';
Query.NOT_LIKE = 'NOT_LIKE';
Query.IN = 'IN';
Query.NOT_IN = 'NOT_IN';
Query.BETWEEN = 'BETWEEN';
Query.NOT_BETWEEN = 'NOT_BETWEEN';
Query.IS_NULL = 'IS_NULL';
Query.IS_NOT_NULL = 'IS_NOT_NULL';
Query.EQ = '=';
Query.NEQ = '<>';
Query.GT = '>';
Query.GTE = '>=';
Query.LT = '<';
Query.LTE = '<=';
Query.ORDER = 'ORDER_BY';
Query.ORDER_ASC = 'ASC';
Query.ORDER_DESC = 'DESC';
Query.GROUP = 'GROUP_BY';